@@ -3,14 +3,18 @@ package Game::HeroesVsAliens;
 use 5.006;
 use strict;
 use warnings;
-our $VERSION = '0.02';
+our $VERSION = '1.00';
 
 use Moo;
 
 use Game::HeroesVsAliens::Mouse;
 use Game::HeroesVsAliens::Cell;
-use Game::HeroesVsAliens::Hero;
-use Game::HeroesVsAliens::Alien;
+use Game::HeroesVsAliens::Hero::Blue;
+use Game::HeroesVsAliens::Hero::Pink;
+use Game::HeroesVsAliens::Alien::One;
+use Game::HeroesVsAliens::Alien::Two;
+use Game::HeroesVsAliens::Alien::Three;
+use Game::HeroesVsAliens::Alien::Four;
 use Game::HeroesVsAliens::Message;
 use Game::HeroesVsAliens::Bonus;
 
@@ -19,6 +23,7 @@ use SDLx::App;
 use SDL::GFX::Primitives;
 use SDLx::Text;
 use SDL::GFX::Primitives;
+use SDLx::Sound;
 use Data::LnArray;
 
 with 'Game::HeroesVsAliens::Role::ResourceDirectory';
@@ -38,21 +43,31 @@ has heroes => (
 	default => sub { Data::LnArray->new; }
 );
 
+has hero_upgrades => (
+	is => 'rw',
+	default => sub {
+		{
+			1 => 'Game::HeroesVsAliens::Hero::Blue',
+			2 => 'Game::HeroesVsAliens::Hero::Pink'
+		}
+	}
+);
+
+has hero_cost => (
+	is => 'rw',
+	default => sub { 100 }
+);
+
+has hero_upgrade_cost => (
+	is => 'rw',
+	default => sub { 100 }
+);
+
 has aliens => (
 	is => 'rw',
 	default => sub { Data::LnArray->new; }
 );
 
-has messages => (
-	is => 'rw',
-	default => sub { Data::LnArray->new; }
-);
-
-has bonuses => (
-	is => 'rw',
-	default => sub { Data::LnArray->new; }
-);
-
 has alien_positions => (
 	is => 'rw',
 	default => sub { Data::LnArray->new; }
@@ -63,6 +78,17 @@ has aliens_interval => (
 	default => sub { 400 }
 );
 
+has messages => (
+	is => 'rw',
+	default => sub { Data::LnArray->new; }
+);
+
+has bonuses => (
+	is => 'rw',
+	default => sub { Data::LnArray->new; }
+);
+
+
 has frame => (
 	is => 'rw',
 	default => sub { 0 }
@@ -93,7 +119,29 @@ has score => (
 	default => sub { 0 }
 );
 
-has winning_score => (
+has stages => (
+	is => 'rw',
+	default => sub {
+		{
+			1 => 'Game::HeroesVsAliens::Alien::One',
+			2 => 'Game::HeroesVsAliens::Alien::Two',
+			3 => 'Game::HeroesVsAliens::Alien::Three',
+			4 => 'Game::HeroesVsAliens::Alien::Four',
+		} 
+	}
+);
+
+has stage => (
+	is => 'rw',
+	default => sub { 1 }
+);
+
+has stage_winning_score => (
+	is => 'rw',
+	default => sub { 500 }
+);
+
+has stage_score => (
 	is => 'rw',
 	default => sub { 500 }
 );
@@ -161,8 +209,6 @@ sub run {
 		)
 	);
 
-	$self->resource_directory;
-
 	$self->ctx->add_event_handler(sub {
 		my $e = shift;
 		if ($e->type == SDL_MOUSEMOTION) {
@@ -192,6 +238,9 @@ sub run {
 
 	$self->create_grid();
 
+	my $snd = SDLx::Sound->new;
+	$snd->play($self->resource_directory . 'resources/background.wav');
+
 	$self->ctx->run();
 }
 
@@ -228,45 +277,114 @@ sub handle_game_status {
 			'Game Over'
 		);
 	}
-	if ($self->score >= $self->winning_score && $self->aliens->length == 0) {
-		$self->draw_text(
-			130,
-			200,
-			100,
-			[255, 255, 255, 255],
-			'Level Complete'
-		);
-		$self->draw_text(
-			130,
-			300,
-			50,
-			[255, 255, 255, 255],
-			'You win with ' . $self->score . ' points'
-		);
-		$self->game_over(1);
+	if ($self->score >= $self->stage_winning_score && $self->aliens->length == 0) {
+		my $next_stage = $self->stage + 1;
+		if ($self->stages->{$next_stage}) {
+			$self->messages->push(Game::HeroesVsAliens::Message->new({
+				value => 'You cleared stage ' . $self->stage,
+				x => 130,
+				y => 200,
+				size => 50,
+				color => [255, 255, 255],
+			}));
+			$self->stage_winning_score($self->score + $self->stage_score);	
+			$self->stage($next_stage);
+			$self->aliens_interval(400);
+		} else {
+			$self->draw_text(
+				130,
+				200,
+				100,
+				[255, 255, 255, 255],
+				'Level Complete'
+			);
+			$self->draw_text(
+				130,
+				300,
+				50,
+				[255, 255, 255, 255],
+				'You win with ' . $self->score . ' points'
+			);
+			$self->game_over(1);
+		}
 	}
 }
 
+sub upgrade_hero {
+	my ($self, $i, $hero) = @_;
+
+	my $next_level = $hero->level + 1;
+	if ($self->hero_upgrades->{$next_level}) {
+		if ($self->resources >= $self->hero_cost) {
+			$self->heroes->splice($i, 1, $self->hero_upgrades->{$next_level}->new({
+				level => $next_level,
+				x => $hero->x,
+				y => $hero->y,
+				width => $hero->width,
+				height => $hero->height,
+			}));
+			$self->resources( $self->resources - $self->hero_cost );
+			$self->messages->push(Game::HeroesVsAliens::Message->new({
+				value => 'Hero upgraded',
+				x => $hero->x,
+				y => $hero->y,
+				size => 20,
+				color => [0, 0, 0],
+			}));
+		} else {
+			$self->messages->push(Game::HeroesVsAliens::Message->new({
+				value => 'Not enough resources',
+				x => $hero->x,
+				y => $hero->y,
+				size => 20,
+				color => [0, 0, 0],
+			}));
+		}
+	} else {
+		$self->messages->push(Game::HeroesVsAliens::Message->new({
+			value => 'Hero is fully upgraded',
+			x => $hero->x,
+			y => $hero->y,
+			size => 20,
+			color => [0, 0, 0],
+		}));
+	}
+}
+
+
 sub add_hero {
 	my $self = shift;
 	my $pos_x = $self->mouse->x - ($self->mouse->x % $self->cell_size) + $self->cell_gap;
 	my $pos_y = $self->mouse->y - ($self->mouse->y % $self->cell_size) + $self->cell_gap;
 	return if ($pos_y < $self->cell_size);
-	for my $hero (@{$self->heroes}) {
+	for (my $i = 0; $i < $self->heroes->length; $i++) {
+		my $hero = $self->heroes->[$i];
 		if ($hero->x == $pos_x && $hero->y == $pos_y) {
+			if ($hero->last_click_frame && $hero->last_click_frame - $self->frame < 100) {
+				$self->upgrade_hero($i, $hero);	
+			} else {
+				$self->messages->push(Game::HeroesVsAliens::Message->new({
+					value => 'click again to upgrade',
+					x => $hero->x,
+					y => $hero->y,
+					size => 20,
+					color => [0, 0, 0],
+				}));
+				$hero->last_click_frame($self->frame);
+			}
 			return;
 		}
 	}
 
-	my $hero_cost = 100;
-	if ($self->resources >= $hero_cost) {
-		$self->heroes->push(Game::HeroesVsAliens::Hero->new(
+	if ($self->resources >= $self->hero_cost) {
+		$self->heroes->push($self->hero_upgrades->{1}->new(
+			level => 1,
 			x => $pos_x,
 			y => $pos_y,
 			width => $self->cell_size - $self->cell_gap * 2,
 			height => $self->cell_size - $self->cell_gap * 2
 		));
-		$self->resources( $self->resources - $hero_cost );
+		$self->resources( $self->resources - $self->hero_cost );
 	}
 }
 
@@ -277,28 +395,30 @@ sub handle_heroes {
 		$heroes->[$i]->draw($self);
 		$heroes->[$i]->update($self);
 
-		if ($self->alien_positions->indexOf($heroes->[$i]->y) != -1) {
-			$heroes->[$i]->shooting(1);
-		} else {
-			$heroes->[$i]->shooting(0);
-		}
-
 		my $aliens = $self->aliens;
-		for (my $j = 0; $j < scalar @{$aliens}; $j++) {
-			if ($heroes->[$i] && $self->collision($heroes->[$i], $aliens->[$j])) {
-				$aliens->[$j]->movement(0);
-				$heroes->[$i]->health($heroes->[$i]->health - 0.2);
-			}
-		}
-
 		if ($heroes->[$i]->health <= 0) {
+			if ($heroes->[$i]->death_sprite->current_loop > 1) {
+				for (my $j = 0; $j < scalar @{$aliens}; $j++) {
+					if ($self->collision($heroes->[$i], $aliens->[$j])) {
+						$aliens->[$j]->movement($aliens->[$j]->speed);
+					}
+				}
+				$heroes->splice($i, 1);
+				$i--;
+			}
+		} else {
+			if ($self->alien_positions->indexOf($heroes->[$i]->y) != -1) {
+				$heroes->[$i]->shooting(1);
+			} else {
+				$heroes->[$i]->shooting(0);
+			}
+
 			for (my $j = 0; $j < scalar @{$aliens}; $j++) {
-				if ($self->collision($heroes->[$i], $aliens->[$j])) {
-					$aliens->[$j]->movement($aliens->[$j]->speed);
+				if ($heroes->[$i] && $self->collision($heroes->[$i], $aliens->[$j])) {
+					$aliens->[$j]->movement(0);
+					$heroes->[$i]->health($heroes->[$i]->health - 0.2);
 				}
 			}
-			$heroes->splice($i, 1);
-			$i--;
 		}
 	}
 }
@@ -344,9 +464,9 @@ sub handle_aliens {
 			$aliens->[$i]->dead(1);
 		}
 	}
-	if ($self->frame % $self->aliens_interval == 0 && $self->score < $self->winning_score) {
+	if ($self->frame % $self->aliens_interval == 0 && $self->score < $self->stage_winning_score) {
 		my $y_pos = int(rand(5) + 1) * $self->cell_size + $self->cell_gap;
-		$self->aliens->push(Game::HeroesVsAliens::Alien->new({
+		$self->aliens->push($self->stages->{int(rand($self->stage) + 1)}->new({
 			y => $y_pos,
 			x => $self->app_width,
 			width => $self->cell_size - $self->cell_gap * 2,
@@ -354,7 +474,8 @@ sub handle_aliens {
 		}));
 		$self->alien_positions->push($y_pos);
 		if ($self->aliens_interval > 10) {
-			$self->aliens_interval($self->aliens_interval - int(rand(40)));
+			my $interval = $self->aliens_interval - int(rand(40));
+			$self->aliens_interval($interval > 10 ? $interval : 10);
 		}
 	}
 }
@@ -375,7 +496,7 @@ sub handle_messages {
 sub handle_bonuses {
 	my ($self) = @_;
 
-	if ($self->frame % 500 == 0 && $self->score < $self->winning_score) {
+	if ($self->frame % 500 == 0 && $self->score < $self->stage_winning_score) {
 		my $y_pos = int(rand(5) + 1) * $self->cell_size + $self->cell_gap;
 		my $x_pos = int(rand(8)) * $self->cell_size + $self->cell_gap;
 		$self->bonuses->push(Game::HeroesVsAliens::Bonus->new({
